package com.shiro.business;

import org.junit.Test;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.text.Collator;
import java.util.*;

/**
 * @作者: 夏容清
 * @创建日期: 2019/1/4 14:28
 * @说明: list通用排序类
 */
public class SortList<E> {

    /**
     * @行参说明: list:需要排序的集合;method:按照哪个方法进行排序(如：id则按该类的id进行排序);
     * sort:排序,可填入的值为:asc,desc;type:排序类型(如：Integer)不写则会自动按属性类型排序
     * @返回值说明: 无
     * @作者:  夏容清
     * @创建日期: 2019/1/5 10:46
     * @功能描述: 自定义通用List排序
     */
    public void sortTo(List<E> list,final String method,final String sort,final String type){
        if(list == null || list.size() < 2) {
            return;
        }
        Collections.sort(list, new Comparator() {
            @Override
            public int compare(Object o1, Object o2) {
                try {
                    Class clazz = o1.getClass();
                    Field field = clazz.getDeclaredField(method); //获取成员变量
                    field.setAccessible(true); //设置成可访问状态
                    String property = field.getType().getName().toLowerCase();
                    String typeName;

                    if("date".equalsIgnoreCase(type)||"timestamp".equalsIgnoreCase(type)){
                        typeName = "java.sql."+type.toLowerCase();//转换成小写
                    } else if("int".equalsIgnoreCase(type)){
                        typeName = type.toLowerCase();
                    } else  if(null != type && !"".equals(type)){
                        typeName = "java.lang."+type.toLowerCase();
                    } else {
                        typeName = field.getType().getName().toLowerCase();
                    }

                    Object v1 = field.get(o1); //获取field的值
                    Object v2 = field.get(o2); //获取field的值

                    boolean ASC_order = (sort == null || "ASC".equalsIgnoreCase(sort));
                    //判断字段数据类型，并比较大小
                    if(typeName.endsWith("string")) {
                        Collator cmp = Collator.getInstance(java.util.Locale.CHINA);
                        String value1 = v1.toString();
                        String value2 = v2.toString();
                        return ASC_order ? cmp.compare(value1, value2) : cmp.compare(value2, value1);
                    } else if(typeName.endsWith("short")) {
                        Short value1 = Short.parseShort(v1.toString());
                        Short value2 = Short.parseShort(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("byte")) {
                        Byte value1 = Byte.parseByte(v1.toString());
                        Byte value2 = Byte.parseByte(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("char")) {
                        Integer value1 = (int)(v1.toString().charAt(0));
                        Integer value2 = (int)(v2.toString().charAt(0));
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.equals("int") || typeName.endsWith("integer")) {
                        if(property.equals(typeName)) {
                            Integer value1 = Integer.parseInt(v1.toString());
                            Integer value2 = Integer.parseInt(v2.toString());
                            return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                        } else{
                            String value1 = v1.toString();
                            String value2 = v2.toString();
                            return ASC_order ? compareTo(value1,value2) : compareTo(value2,value1);
                        }
                    } else if(typeName.endsWith("long")) {
                        Long value1 = Long.parseLong(v1.toString());
                        Long value2 = Long.parseLong(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("float")) {
                        Float value1 = Float.parseFloat(v1.toString());
                        Float value2 = Float.parseFloat(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("double")) {
                        Double value1 = Double.parseDouble(v1.toString());
                        Double value2 = Double.parseDouble(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("boolean")) {
                        Boolean value1 = Boolean.parseBoolean(v1.toString());
                        Boolean value2 = Boolean.parseBoolean(v2.toString());
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("date")) {
                        Date value1 = (Date)(v1);
                        Date value2 = (Date)(v2);
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else if(typeName.endsWith("timestamp")) {
                        Timestamp value1 = (Timestamp)(v1);
                        Timestamp value2 = (Timestamp)(v2);
                        return ASC_order ? value1.compareTo(value2) : value2.compareTo(value1);
                    } else {
                        //调用对象的compareTo()方法比较大小
                        Method method = field.getType().getDeclaredMethod("compareTo", new Class[]{field.getType()});
                        method.setAccessible(true); //设置可访问权限
                        int result  = (Integer)method.invoke(v1, new Object[]{v2});
                        return ASC_order ? result : result*(-1);
                    }
                } catch (Exception e) {
                    String err = e.getLocalizedMessage();
                    System.out.println(err);
                    e.printStackTrace();
                }
                return 0; //未知类型，无法比较大小
            }
        });
    }
	
	private int getNum(String s) {
        String n = new String();
        for(int i = s.length() - 1; i >= 0 && s.charAt(i) >= '0' && s.charAt(i) <= '9'; i--)
            n = s.charAt(i) + n;
        return Integer.valueOf(n).intValue();
    }
	
	public int compareTo(String s1, String s2) {
        return getNum(s1) - getNum(s2);
    }

    public SortList() {
        super();
    }


    //执行示例
    @Test
    public void sortTest(){
        List<News> list=new ArrayList<>();

        News news1=new News();
        news1.setHits(1);
        news1.setId(1);
        news1.setTitle("张三9");
        list.add(news1);

        News news2=new News();
        news2.setHits(7);
        news2.setId(2);
        news2.setTitle("李四10");
        list.add(news2);

        News news3=new News();
        news3.setHits(3);
        news3.setId(3);
        news3.setTitle("王五4");
        list.add(news3);

        News news4=new News();
        news4.setHits(5);
        news4.setId(4);
        news4.setTitle("赵六1");
        list.add(news4);

//        SortList<News> sortListIds = new SortList<>();
//        //按userId排序
//        sortListIds.sortTo(list, "id", "desc","Integer");
//        System.out.println("--------按userId倒序------------------");
//        for(News news :list){
//            System.out.println(news);
//        }

        SortList<News> sortListTitle = new SortList<>();
        //按title排序
        sortListTitle.sortTo(list, "title", "desc","int");
        System.out.println("--------按title倒序------------------");
        for(News news :list){
            System.out.println(news);
        }
    }

}
